using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace TestJumble
{
	/// <summary>
	/// This program solves Jumbles real quick.
	/// Run the program, and type in a jumbled word.
	/// Depends on a dictionary called 'cedt.dic' in the current directory.
	/// </summary>
	class Program
	{
		static StreamReader sr = null;
        static Dictionary<string, string> dic;
        static List<string> wordgamewords = new List<string>();
        static ConsoleColor defaultcolor;

		public static void Main() {

			//The StreamReader that reads in words.
			sr = new StreamReader("cedt.dic");

			//The name-value pairs of sorted strings and actual strings
			dic = new Dictionary<string,string>();

			string s = "", sorteds="";

			//Build the Dictionary.
			//Consider caching this to a file for reducing startup time
			Console.Write("Loading Dictionary...");
			while((s = getWord()) != null) {
				sorteds = sort(s);
				if(!dic.ContainsKey(sorteds)) {
					dic.Add(sorteds, s);
					//Console.Write(sorteds + "\t" + s);
				}
				else {
					if(s.Trim().Length > 0) {//if it is not all whitespace
						dic[sorteds] = dic[sorteds] + "," + s;
					}
				}
			}
			Console.WriteLine("\rDictionary Loaded.      ");
			Console.WriteLine("Type in a jumbled word, and I will try to guess it. \r\nType in exit to quit.");

            //findAllLoveCombos();


			//Till the user shouts 'Stop', keep solving jumbles
			defaultcolor = Console.ForegroundColor;
			while(true) {
                Console.Write("> ");
				s = Console.ReadLine().ToLower();
				if(s.Equals("exit")) return;
				sorteds = sort(s);
                Console.Write("Exact match(es): ");
				Console.ForegroundColor = ConsoleColor.Red;
				try {
					string st = dic[sorteds];
					string[] sarr = st.Split(new char[] { ',' });
					Console.Write(sarr[0]);
					for(int i = 0; i < sarr.Length - 1; i++) {
						Console.Write(", " + sarr[i + 1]);
					}
					Console.WriteLine();
				}
				catch {
					Console.WriteLine("Word not found");
				}

                //now also find out the word-game words from this word
                findWordGameWords(sorteds);


				Console.ForegroundColor = defaultcolor;
			}

		}

        private static void findAllLoveCombos() {
            for (int i = 0; i < 26; i++) {
                for (int j = 0; j < 26; j++) {
                    char ch1 = (char)('a' + i);
                    char ch2 = (char)('a' + j);
                    string s = "love" + ch1 + ch2;
                    string sorteds = sort(s);
                    try {
                        string st = dic[sorteds];
                        string[] sarr = st.Split(new char[] { ',' });
                        Console.Write(sarr[0]);
                        for (int kk = 0; kk < sarr.Length - 1; kk++) {
                            Console.Write(", " + sarr[kk + 1]);
                        }
                        Console.WriteLine();
                    }
                    catch {
                    }
                }
            }


        }

        private static void findWordGameWords(string sorteds) {
            wordgamewords.Clear();
            
            //create strings from all possible combinations.
            int n = twoToPower(sorteds.Length);
            for (int i = 0; i < n; i++) {
                string s = chooseChars(sorteds, i);
                if (s.Length < 3) continue;
                if(dic.ContainsKey(s)){
                    string st = dic[s];
					string[] sarr = st.Split(new char[] { ',' });
                    for (int j = 0; j < sarr.Length; j++) {
                        if (!wordgamewords.Contains(sarr[j]))
                            wordgamewords.Add(sarr[j]);
                    }
                }
            }

            Console.ForegroundColor = defaultcolor;
            Console.Write("Matches from letters of the word ("+wordgamewords.Count+"): ");
            Console.ForegroundColor = ConsoleColor.Red;

            if (wordgamewords.Count < 1) return;

            Console.Write(wordgamewords[0]);
            for (int i = 0; i < wordgamewords.Count - 1; i++) {
                Console.Write(", " + wordgamewords[i + 1]);
            }

            Console.WriteLine();

        }

        private static string chooseChars(string sorteds, int i) {
            string s = "";
            int index = sorteds.Length - 1;
            while (i>0) {
                if (i % 2 == 1) {
                    s = sorteds[index] + s;
                }
                index--;
                i /= 2;
            }
            return s;
        }

        private static int twoToPower(int n) {
            int reply = 1;
            for (int i = 0; i < n; i++) {
                reply *= 2;
            }
            return reply;
        }

		/// <summary>
		///	From a text-file, read one space delimited word and return it.
		/// </summary>
		/// <returns>The next word in the File</returns>
		private static string getWord() {
			int i = 0;
			char ch = ' ';
			string reply = "";
			while(true) {
				i = sr.Read();
				if(i == -1) return null;
				ch = (char)i;
				if(char.IsWhiteSpace(ch)) break;
				reply += ch;
			}
			//if("".Equals(reply)) return null;
			return reply;
		}

		/// <summary>
		/// Sorts the characters in the input string and returns a string
		/// of those charaters sorted.
		/// </summary>
		/// <param name="s">The string to sort</param>
		/// <returns>The string having the characters sorted</returns>
		private static string sort(string s) {
			//Make a buffer
			char[] so = new char[s.Length];
			char ch;
			int i = 0, j = 0;
			//Fill the buffer with characters
			for(i = 0; i < s.Length; i++) {
				so[i] = s[i];
			}
			//Ripple sort the characters
			for(i = 0; i < s.Length - 1; i++) {
				for(j = 0; j < s.Length - i-1; j++) 
				{
					if(so[j] > so[j + 1]) {
						ch = so[j];
						so[j] = so[j + 1];
						so[j + 1] = ch;
					}
				}
			}
			//Build the return string
			string st = "";
			foreach(char chh in so)
				st += chh;
			return st;
		}

	}
}
